/************************************************************************
 *
 * SOS - Small Operating System
 *
 * This is an RTOS for TI MSP430 microcontrollers family.
 *
 * NOTICE: Copyright (C) 2002, 2003, 2004 by Oleg Skydan
 *
 * Files means any text, html, source or binary files that 
 * came with this copyright notice.
 *
 * You may use, embed, modify or distribute the contents
 * of the file(s) in any way you see fit as long as you
 * include this copyright notice in the resulting file(s).
 *
 * Any commercial or industrial use, except of SPECIAL EXCEPTION, 
 * including publishing or distributing SOS in any form IS 
 * PROHIBITED, and requires the written authorization of the author. 
 *
 * SPECIAL EXCEPTION:
 * If you choose to link SOS with other files to produce an
 * executable, this does not by itself cause the resulting 
 * executable to be covered by this license. Your use of the 
 * executable is in no way restricted on account of linking 
 * the SOS code with it. 
 *
 * SOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; WITH NO RESPONSIBILITY 
 * assumed by the author for bug fixes, support, or 
 * DAMAGES OF ANY SORT arising from the use of the SOS. 
 * 
 *
 * The program author can be contacted by E-mail at skidan@mail.ints.net
 *
 * For more details on the background or use of the program, please visit
 * http://users.ints.net/skidan/SOS
 *
 *
 * Critical section routines
 ****************************************************************************/

#include "Kernel.h"


void KEnterCS(cs_t* cs)
{
	if(*cs==0)
	{  //First call, critical section is free
		*cs=GetTID()|CS_STEP;
		return;
	}
	if((*cs & CS_TID)==GetTID())
	{  //Critical section is occupied by current task, increase counter
		*cs+=CS_STEP;
		return;
	}
	//Wait for freeing critical section
	R_TASK.state|=TS_BLOCKED;
	KReschedule();
}

void KLeaveCS(cs_t* cs)
{
	*cs-=CS_STEP;
	if((*cs & CS_COUNTER)!=0)return;
	*cs=0; //free critical section
	int t=0;
	//Find task with max priority blocked by this critical section 
	//Note that Idle task (tid==0) is always ready
	for(int i=1;i<MAX_TASKS;i++)
	{
		if(Tasks[i].priority>Tasks[t].priority)
		{
			uint8_t state=Tasks[i].state;
			if(state==(TS_READY|TS_BLOCKED) && T_CS_PTR(i)==cs)t=i;
		}
	}
	
	if(t!=0)
	{
		*cs=t|CS_STEP;
		Tasks[t].state=TS_READY;
		KReschedule();
	}
}

void EnterCS(cs_t* cs)
{
	ENTER();
	KEnterCS(cs);
	LEAVE();
}

void LeaveCS(cs_t* cs)
{
	ENTER();
	KLeaveCS(cs);
	LEAVE();
}
