/************************************************************************
 *
 * SOS - Small Operating System
 *
 * This is an RTOS for TI MSP430 microcontrollers family.
 *
 * NOTICE: Copyright (C) 2002, 2003, 2004 by Oleg Skydan
 *
 * Files means any text, html, source or binary files that 
 * came with this copyright notice.
 *
 * You may use, embed, modify or distribute the contents
 * of the file(s) in any way you see fit as long as you
 * include this copyright notice in the resulting file(s).
 *
 * Any commercial or industrial use, except of SPECIAL EXCEPTION, 
 * including publishing or distributing SOS in any form IS 
 * PROHIBITED, and requires the written authorization of the author. 
 *
 * SPECIAL EXCEPTION:
 * If you choose to link SOS with other files to produce an
 * executable, this does not by itself cause the resulting 
 * executable to be covered by this license. Your use of the 
 * executable is in no way restricted on account of linking 
 * the SOS code with it. 
 *
 * SOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; WITH NO RESPONSIBILITY 
 * assumed by the author for bug fixes, support, or 
 * DAMAGES OF ANY SORT arising from the use of the SOS. 
 * 
 *
 * The program author can be contacted by E-mail at skidan@mail.ints.net
 *
 * For more details on the background or use of the program, please visit
 * http://users.ints.net/skidan/SOS
 *
 * Events routines
 ****************************************************************************/

#include "Kernel.h"

//Events state
uint32_t Events;
//Events mode (if corresponding bit is 1-auto reset/0-manual reset event)
uint32_t EventsMode=EVENTS_MODE;


void KWaitEvents(uint32_t mask,uint8_t mode)
{
	uint32_t set=Events & mask;//Events that already set
	if((mode==WAIT_ALL && set==mask)
	|| (mode==WAIT_ANY && set!=0))
	{
		T_RES32(GetTID())=set;
		KResetEvents(set & EventsMode);
		return;
	}
	KReschedule();
	R_TASK.state|=mode;
}

void KSetEvents(uint32_t mask)
{
	//If we have no new event(s), continue
	if((Events & mask)!=mask)
	{
		Events|=mask;
		int t;
		do{
			//Find task with max priority pending for events 
			//Note that Idle task (tid==0) is always ready
			t=0;
			for(int i=1;i<MAX_TASKS;i++)
			{
				if(Tasks[i].priority>Tasks[t].priority)
				{
					uint32_t pending=T_EVENTS_MASK(i);
					uint32_t set=pending & Events;
					uint8_t state=Tasks[i].state;
					if((state==(TS_READY|TS_PENDINGANY) && set!=0)
						|| (state==(TS_READY|TS_PENDINGALL) && set==pending))t=i;
				}
			}
			
			if(t!=0)
			{
				uint32_t set=T_EVENTS_MASK(t) & Events;
				T_RES32(t)=set; //WaitEvents return value
				KResetEvents(set & EventsMode);//Clear auto reset events, consumed by task
				Tasks[t].state=TS_READY;
				KReschedule();
			}
		}while(t!=0 && (Events & mask));
	}
}

void ResetEvents(uint32_t mask)
{
	KResetEvents(mask);
}


uint32_t TryEvents(uint32_t mask,uint8_t mode)
{
	uint32_t set=Events & mask;//Events that already set
	if((mode==WAIT_ALL && set==mask)
	|| (mode==WAIT_ANY && set!=0))
	{
		KResetEvents(set & EventsMode);
		return set;
	}
	return 0;
}

uint32_t WaitEvents(uint32_t mask,uint8_t mode)
{
	ENTER();
	KWaitEvents(mask,mode);
	LEAVE();
}

void SetEvents(uint32_t mask)
{
	ENTER();
	KSetEvents(mask);
	LEAVE();
}

void SetAutoEvents(uint32_t a)
{
	KSetAutoEvents(a);
}

void SetManualEvents(uint32_t m)
{
	KSetManualEvents(m);
}
