/************************************************************************
 *
 * SOS - Small Operating System
 *
 * This is an RTOS for TI MSP430 microcontrollers family.
 *
 * NOTICE: Copyright (C) 2002, 2003, 2004 by Oleg Skydan
 *
 * Files means any text, html, source or binary files that 
 * came with this copyright notice.
 *
 * You may use, embed, modify or distribute the contents
 * of the file(s) in any way you see fit as long as you
 * include this copyright notice in the resulting file(s).
 *
 * Any commercial or industrial use, except of SPECIAL EXCEPTION, 
 * including publishing or distributing SOS in any form IS 
 * PROHIBITED, and requires the written authorization of the author. 
 *
 * SPECIAL EXCEPTION:
 * If you choose to link SOS with other files to produce an
 * executable, this does not by itself cause the resulting 
 * executable to be covered by this license. Your use of the 
 * executable is in no way restricted on account of linking 
 * the SOS code with it. 
 *
 * SOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; WITH NO RESPONSIBILITY 
 * assumed by the author for bug fixes, support, or 
 * DAMAGES OF ANY SORT arising from the use of the SOS. 
 * 
 *
 * The program author can be contacted by E-mail at skidan@mail.ints.net
 *
 * For more details on the background or use of the program, please visit
 * http://users.ints.net/skidan/SOS
 *
 * Header file for system device drivers use
 ****************************************************************************/

#ifndef __KERNEL_H_INCLUDED__
#define __KERNEL_H_INCLUDED__

#include "OS.h"

#define osinterrupt(v) void __attribute__((interrupt(v),naked))

#define ENTER() asm volatile("push R2		 \n\t"\
							        "dint			 \n\t"\
                             "call #__Enter \n\t"\
                             ".if .L__FrameSize_"__FUNCTION__" \n\t"\
                             "sub #.L__FrameSize_"__FUNCTION__",r1 \n\t"\
                             ".endif " : : : "r11", "r10");

#define ENTERINT() asm volatile("call #__Enter \n\t"\
                                ".if .L__FrameSize_"__FUNCTION__" \n\t"\
                                "sub #.L__FrameSize_"__FUNCTION__",r1 \n\t"\
                                ".endif " : : : "r11", "r10");

#define LEAVE() asm volatile(".if .L__FrameSize_"__FUNCTION__" \n\t"\
                             "add #.L__FrameSize_"__FUNCTION__",r1 \n\t"\
                             ".endif \n\t"\
                             "br #__Leave	\n\t" : : : "r11" )	


#define R_TASK Tasks[GetTID()]

#define T_RES32(t) ((uint32_t*)Tasks[t].stack)[0]  //32 bit result in R15:R14

#define T_RES8(t) Tasks[t].stack[1]  //Result in R15

#define T_RES16(t) Tasks[t].stack[1]  //Result in R15

#define T_CS_PTR(t) (cs_t*)Tasks[t].stack[1] //Access to critical section pointer
											 //(arguments of EnterCS,LeaveCS)
#define T_EVENTS_MASK(t) ((uint32_t*)Tasks[t].stack)[0]




typedef struct
{
	uint16_t *stack;
	uint8_t state;
	uint8_t priority;
}TCB;

extern TCB Tasks[];



#define KReschedule() ({ extern uint8_t Reschedule; Reschedule=1; }) 

void KSetTaskPriority(tid_t tid,uint8_t p);
void KTerminateTask(tid_t tid);

//Suspend task execution
void KSuspendTask(tid_t tid);
//Resume task execution
void KResumeTask(tid_t tid);


void KWaitEvents(uint32_t mask,uint8_t mode);
void KSetEvents(uint32_t mask);

#define KResetEvents(mask) ({ extern uint32_t Events; \
							  asm volatile("bic %A1,%A0	\n\t"\
										   "bic %B1,%B0	\n\t"\
										   : "=mr" ((uint32_t)Events) \
										   : "imr" ((uint32_t)mask), "0" ((uint32_t)Events));\
							})


#define KSetAutoEvents(e) ({ extern uint32_t EventsMode; \
						     asm volatile("bis %A1,%A0	\n\t"\
										  "bis %B1,%B0	\n\t"\
										  : "=mr" ((uint32_t)EventsMode) \
										  : "imr" (e), "0" ((uint32_t)EventsMode));\
						   })

#define KSetManualEvents(e)  ({ extern uint32_t EventsMode; \
						        asm volatile("bic %A1,%A0	\n\t"\
											 "bic %B1,%B0	\n\t"\
											 : "=mr" ((uint32_t)EventsMode) \
											 : "imr" (e), "0" ((uint32_t)EventsMode));\
						     })


void KEnterCS(cs_t* cs);
void KLeaveCS(cs_t* cs);

#endif
