/************************************************************************
 *
 * SOS - Small Operating System
 *
 * This is an RTOS for TI MSP430 microcontrollers family.
 *
 * NOTICE: Copyright (C) 2002, 2003, 2004 by Oleg Skydan
 *
 * Files means any text, html, source or binary files that 
 * came with this copyright notice.
 *
 * You may use, embed, modify or distribute the contents
 * of the file(s) in any way you see fit as long as you
 * include this copyright notice in the resulting file(s).
 *
 * Any commercial or industrial use, except of SPECIAL EXCEPTION, 
 * including publishing or distributing SOS in any form IS 
 * PROHIBITED, and requires the written authorization of the author. 
 *
 * SPECIAL EXCEPTION:
 * If you choose to link SOS with other files to produce an
 * executable, this does not by itself cause the resulting 
 * executable to be covered by this license. Your use of the 
 * executable is in no way restricted on account of linking 
 * the SOS code with it. 
 *
 * SOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; WITH NO RESPONSIBILITY 
 * assumed by the author for bug fixes, support, or 
 * DAMAGES OF ANY SORT arising from the use of the SOS. 
 * 
 *
 * The program author can be contacted by E-mail at skidan@mail.ints.net
 *
 * For more details on the background or use of the program, please visit
 * http://users.ints.net/skidan/SOS
 *
 *  Timer module based on MSP430 Timer B hardware module
 ****************************************************************************/

#include "Kernel.h"

static uint16_t Timeout[TIMERS_COUNT];
static uint8_t Active;
static uint8_t Suicidal;


#define TBCCR ((uint16_t*)TBCCR0_)
#define TBCCTL ((uint16_t*)TBCCTL0_)

#define SetTimerEvent(ch)     asm volatile("mov.b %0,R15 \n\t"\
                                  "jmp TimerEvent  \n\t"\
                                  : :"imr" ((uint8_t)ch));


osinterrupt(TIMERB0_VECTOR) TimerB0(void)
{
   ENTERINT();
   SetTimerEvent(0);
}

osinterrupt(TIMERB1_VECTOR) TimerB1(void)
{
   ENTERINT();
   uint8_t ch=TBIV>>1;
   if(ch>0 && ch<TIMERS_COUNT)SetTimerEvent(ch);
   LEAVE();
}

void __attribute__((naked)) TimerEvent(uint8_t ch)
{
   if(Timeout[ch]!=0)   //If this is a cyclic timer, set up for the next period
   {
      TBCCR[ch]+=Timeout[ch];
   }
   else                 //If this is a one-shoot, just turn the interrupt off
   {
      TBCCTL[ch]&=~CCIE;
      TBCCTL[ch]&=~CCIFG;
   }
   timer_t timer=(1<<ch);
   KSetEvents(timer);
   if(Suicidal & timer)Active&=~timer;
   LEAVE();
}


void SetTimer(timer_t timer,uint16_t t,uint16_t cyc)
{
   timer_t tmp=timer;
   int i=0;
   while(tmp & 0x01)
   {
      tmp>>=1;
      i++;
   }
   KResetEvents(timer);
   Timeout[i]=cyc;
   TBCTL&=~MC1;
   TBCCR[i]=TBR+t;
   TBCTL|=MC1;
   TBCCTL[i]&=~CCIFG;
   TBCCTL[i]|=CCIE;
}

timer_t CreateTimer(uint8_t tt,uint16_t t,uint16_t cyc)
{
   uint8_t a=Active;
   for(int i=0;i<TIMERS_COUNT;i++)
   {
      if(!(a & 0x01))
      {
         Timeout[i]=cyc;
         TBCTL&=~MC1;
         TBCCR[i]=TBR+t;
         TBCTL|=MC1;
         TBCCTL[i]|=CCIE;
         TBCCTL[i]&=~CCIFG;
         timer_t timer=(1<<i);
         KResetEvents(timer);
         if(tt & TT_SYNC)KSetAutoEvents(timer);
         else KSetManualEvents(timer);
         if(tt & TT_SUICIDAL)Suicidal|=timer;
         else Suicidal&=~timer;
         Active|=timer;
         return timer;
      }
      a>>=1;
   }
   return 0;
}

void KillTimer(timer_t t)
{
   Active&=~t;
}

void Sleep(uint16_t t)
{
   WaitEvents(CreateTimer(TT_PULSE,t,0),WAIT_ALL);
}


